<?php
/**
 * Returns the post types that have Layout Options.
 *
 * @return array
 */
function gutenbee_get_single_layout_options_post_types() {
	return apply_filters( 'gutenbee_single_layout_options_post_types', array(
		'post',
		'page',
	) );
}

add_action( 'admin_init', 'gutenbee_setup_single_layout_options_metabox' );
function gutenbee_setup_single_layout_options_metabox() {
	foreach ( gutenbee_get_single_layout_options_post_types() as $post_type ) {
		add_meta_box( 'gutenbee-single-layout-options', esc_html__( 'Layout Options', 'gutenbee' ), 'gutenbee_single_layout_options_metabox', $post_type, 'side', 'default' );
	}

	add_action( 'save_post', 'gutenbee_single_layout_options_save_post' );
}

function gutenbee_single_layout_options_metabox( $object, $box ) {
	// Nonce generated inside gutenbee_prepare_metabox()
	gutenbee_prepare_metabox( $object->post_type );

	gutenbee_metabox_dropdown( 'header_layout_type', array(
		'title'   => __( 'Header type:', 'gutenbee' ),
		'default' => '',
		'choices' => gutenbee_header_layout_type_meta_choices(),
	) );

	$pt_name = get_post_type_object( $object->post_type )->labels->singular_name;
	gutenbee_metabox_checkbox( 'gutenbee_hide_single_title', array(
		/* translators: %s is a post type's singular name, e.g. post, page, event, etc. */
		'title'         => sprintf( __( 'Hide title when viewing this %s.', 'gutenbee' ), $pt_name ),
		'checked_value' => 1,
		'default'       => '',
	) );
}

function gutenbee_single_layout_options_save_post( $post_id ) {
	// Nonce verification is being done inside gutenbee_can_save_meta()
	// phpcs:disable WordPress.Security.NonceVerification
	if ( ! gutenbee_can_save_meta( get_post_type( $post_id ) ) ) {
		return;
	}

	if ( ! in_array( get_post_type( $post_id ), gutenbee_get_single_layout_options_post_types(), true ) ) {
		return;
	}


	if ( isset( $_POST['header_layout_type'] ) ) {
		update_post_meta( $post_id, 'header_layout_type', gutenbee_sanitize_header_layout_type_meta( $_POST['header_layout_type'] ) );
	}

	update_post_meta( $post_id, 'gutenbee_hide_single_title', isset( $_POST['gutenbee_hide_single_title'] ) );

	// phpcs:enable
}



//
// Helpers
//
/**
 * Prepares the metabox with the required nonces.
 *
 * @param $post_type string The post type that the metabox is being displayed at.
 */
function gutenbee_prepare_metabox( $post_type ) {
	wp_nonce_field( basename( __FILE__ ), $post_type . '_nonce' );
}

/**
 * Determined whether the metaboxes' values can be saved.
 * Checks nonces, capabilities, etc.
 *
 * @param $post_type string The post type that the metabox is being displayed at.
 *
 * @return bool True if the metaboxe's value should be saved, false otherwise.
 */
function gutenbee_can_save_meta( $post_type ) {
	global $post;

	if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
		return false;
	}

	if ( ! isset( $_POST[ $post_type . '_nonce' ] ) || ! wp_verify_nonce( sanitize_key( wp_unslash( $_POST[ $post_type . '_nonce' ] ) ), basename( __FILE__ ) ) ) {
		return false;
	}

	if ( isset( $_POST['post_view'] ) && 'list' === $_POST['post_view'] ) {
		return false;
	}

	if ( ! isset( $_POST['post_type'] ) || $post_type !== $_POST['post_type'] ) {
		return false;
	}

	$post_type_obj = get_post_type_object( $post->post_type );
	if ( ! current_user_can( $post_type_obj->cap->edit_post, $post->ID ) ) {
		return false;
	}

	return true;
}

/**
 * Displays a dropdown field for use in single post metaboxes.
 *
 * @param string $fieldname   The dropdown's name attribute.
 * @param array  $params      {
 *      Array of dropdown parameters.
 *
 *      @type string  $title       The label text.
 *      @type string  $description Additional description text.
 *      @type string  $default     The default value of the control. Applies on new posts, when there are no existing post meta.
 *      @type array   $choices     Value => Label array of the dropdown's choices.
 * }
 */
function gutenbee_metabox_dropdown( $fieldname, $params = array() ) {
	$params = (array) $params;

	$defaults = array(
		'title'       => '',
		'description' => '',
		'default'     => '',
		'choices'     => array(),
	);

	$params = wp_parse_args( $params, $defaults );

	if ( array_key_exists( 'value', $params ) ) {
		$value = $params['value'];
	} else {
		$custom_keys = get_post_custom_keys( get_the_ID() );

		if ( is_array( $custom_keys ) && in_array( $fieldname, $custom_keys, true ) ) {
			$value = get_post_meta( get_the_ID(), $fieldname, true );
		} else {
			$value = $params['default'];
		}
	}

	?>
	<div class="gutenbee-side-setting-wrap">
		<?php if ( $params['title'] || $params['description'] ) : ?>
			<label class="gutenbee-side-setting-labels" for="<?php echo esc_attr( $fieldname ); ?>">
				<?php if ( $params['title'] ) : ?>
					<span class="gutenbee-side-setting-label"><?php echo esc_html( $params['title'] ); ?></span>
				<?php endif; ?>
				<?php if ( $params['description'] ) : ?>
					<span class="gutenbee-side-setting-description"><?php echo wp_kses( $params['description'], gutenbee_get_allowed_tags() ); ?></span>
				<?php endif; ?>
			</label>
		<?php endif; ?>

		<div class="gutenbee-side-setting-control">
			<select id="<?php echo esc_attr( $fieldname ); ?>" name="<?php echo esc_attr( $fieldname ); ?>">
				<?php foreach ( $params['choices'] as $opt_value => $opt_title ) : ?>
					<option value="<?php echo esc_attr( $opt_value ); ?>" <?php selected( $value, $opt_value ); ?>><?php echo wp_kses( $opt_title, 'strip' ); ?></option>
				<?php endforeach; ?>
			</select>
		</div>
	</div>
	<?php
}

/**
 * Displays a checkbox field for use in single post metaboxes.
 *
 * @param string $fieldname   The checkbox's name attribute.
 * @param array  $params      {
 *      Array of dropdown parameters.
 *
 *      @type string  $title         The label text.
 *      @type string  $description   Additional description text.
 *      @type string  $checked_value Form value when the checkbox is checked. Default: 1
 *      @type string  $default       The default value of the control. Applies on new posts, when there are no existing post meta.
 * }
 */
function gutenbee_metabox_checkbox( $fieldname, $params = array() ) {
	$defaults = array(
		'title'         => '',
		'description'   => '',
		'checked_value' => 1,
		'default'       => '',
	);

	$params = wp_parse_args( $params, $defaults );

	if ( array_key_exists( 'value', $params ) ) {
		$value = $params['value'];
	} else {
		$custom_keys = get_post_custom_keys( get_the_ID() );

		if ( is_array( $custom_keys ) && in_array( $fieldname, $custom_keys, true ) ) {
			$value = get_post_meta( get_the_ID(), $fieldname, true );
		} else {
			$value = $params['default'];
		}
	}

	?>
	<div class="gutenbee-side-setting-wrap">
		<div class="gutenbee-side-setting-control-checkbox">
			<?php if ( $params['title'] ) : ?>
				<label class="gutenbee-side-setting-checkbox-label" for="<?php echo esc_attr( $fieldname ); ?>">
					<input
						type="checkbox"
						id="<?php echo esc_attr( $fieldname ); ?>"
						name="<?php echo esc_attr( $fieldname ); ?>"
						<?php checked( $value, $params['checked_value'] ); ?>
					/>
					<?php if ( $params['title'] ) : ?>
						<span class="gutenbee-side-setting-label"><?php echo esc_html( $params['title'] ); ?></span>
					<?php endif; ?>
				</label>
			<?php endif; ?>

			<?php if ( $params['description'] ) : ?>
				<span class="gutenbee-side-setting-description"><?php echo wp_kses( $params['description'], gutenbee_get_allowed_tags() ); ?></span>
			<?php endif; ?>
		</div>
	</div>
	<?php
}
