<?php
/**
 * Modify the stylesheet path if needed (non-debug modes).
 */
add_filter( 'stylesheet_uri', 'gutenbee_stylesheet_uri', 10, 2 );
function gutenbee_stylesheet_uri( $stylesheet_uri, $stylesheet_dir_uri ) {
	if ( ! is_child_theme() ) {
		$suffix         = gutenbee_scripts_styles_suffix();
		$stylesheet_uri = preg_replace( '/\.css$/', "{$suffix}.css", $stylesheet_uri );
	}

	return $stylesheet_uri;
}

/**
 * Determines whether concatenated scripts (a.k.a. packaged) can be used.
 * (i.e. replace the individually enqueued scripts with the theme's pre-packaged version).
 *
 * @return bool
 */
function gutenbee_should_replace_concatenated_scripts() {
	$return   = false;
	$suffix   = gutenbee_scripts_styles_suffix();
	$option   = get_theme_mod( 'utilities_assets_minify_pack', gutenbee_customizer_defaults( 'utilities_assets_minify_pack' ) );
	$is_child = is_child_theme();

	// Don't do anything if a child theme is active, or the child theme will need to have a minified css file.
	// The parent (this) theme has enqueued its stylesheet file appropriately, as 'gutenbee-style-parent'.
	// Also, only apply when there is a suffix. It may be force-disabled by having enabled WP_DEBUG or SCRIPT_DEBUG.
	if ( ! is_child_theme() && ! empty( $suffix ) && 'minify_pack' === $option ) {
		$return = true;
	}

	return apply_filters( 'gutenbee_should_replace_concatenated_scripts', $return, $suffix, $option, $is_child );
}

add_action( 'wp', 'gutenbee_replace_concatenated_scripts', 50 );
/**
 * Replaces some individually enqueued scripts with the theme's pre-packaged version, if applicable.
 */
function gutenbee_replace_concatenated_scripts() {
	if ( ! gutenbee_should_replace_concatenated_scripts() ) {
		return;
	}

	$suffix      = gutenbee_scripts_styles_suffix();
	$main_suffix = '.pkg' . $suffix;

	$json          = false;
	$data          = false;
	$manifest_path = get_theme_file_path( '/manifest.json' );

	if ( is_readable( $manifest_path ) && is_file( $manifest_path ) ) {
		// Since file_get_contents() is not allowed, and WP_Filesystem is an overkill for our needs (which
		// does a file_get_contents() itself anyway), we use file() instead. It is no different than require()ing
		// or include()ing the file and saving its contents into an output buffer.
		// Related slack discussion: https://wordpress.slack.com/archives/C02RP4Y3K/p1538832288000100
		$json = implode( '', file( $manifest_path ) );

		$data = json_decode( $json );
	}

	if ( false === $json || is_null( $data ) || ! isset( $data->concat ) ) {
		return;
	}

	if ( ! empty( $data->concat->styles ) ) {
		$styles    = $data->concat->styles;
		$wp_styles = wp_styles();

		foreach ( $styles as $handle => $path ) {
			$style = $wp_styles->registered[ $handle ];
			wp_deregister_style( $handle );
			wp_register_style( $handle, false, $style->deps, $style->ver );
		}

		// Main stylesheet.
		if ( is_child_theme() ) {
			$handle = 'gutenbee-style-parent';
		} else {
			$handle = 'gutenbee-style';
		}

		$style = $wp_styles->registered[ $handle ];
		$uri   = preg_replace( '/' . preg_quote( $suffix, '/' ) . '\.css$/', "{$main_suffix}.css", $style->src );
		wp_deregister_style( $handle );
		wp_register_style( $handle, $uri, $style->deps, $style->ver );
	}

	if ( ! empty( $data->concat->scripts ) && file_exists( get_theme_file_path( "/js/scripts{$main_suffix}.js" ) ) ) {
		$scripts    = $data->concat->scripts;
		$wp_scripts = wp_scripts();

		foreach ( $scripts as $handle => $path ) {
			$script    = $wp_scripts->registered[ $handle ];
			$in_footer = $wp_scripts->get_data( $handle, 'group' );
			// Use of wp_deregister_script() is blocked by the theme-check plugin, however since we are not deregistering
			// any core scripts (only theme ones), we should be able to do that, so we need to work around this restriction,
			// by calling $wp_scripts->remove() directly.
			$wp_scripts->remove( $handle );
			wp_register_script( $handle, false, $script->deps, $script->ver, $in_footer );
		}

		$script = $wp_scripts->registered['gutenbee-front-scripts'];
		$wp_scripts->remove( 'gutenbee-front-scripts' );
		wp_register_script( 'gutenbee-front-scripts', get_theme_file_uri( "/js/scripts{$main_suffix}.js" ), $script->deps, $script->ver, $in_footer );
	}
}

/**
 * Add a pingback url auto-discovery header for singularly identifiable articles.
 */
add_action( 'wp_head', 'gutenbee_pingback_header' );
function gutenbee_pingback_header() {
	if ( is_singular() && pings_open() ) {
		echo '<link rel="pingback" href="', esc_url( get_bloginfo( 'pingback_url' ) ), '">';
	}
}

/**
 * Add classes on the <body> element.
 */
add_filter( 'body_class', 'gutenbee_body_class', 10, 2 );
function gutenbee_body_class( $classes, $class ) {
	if ( 'boxed' === get_theme_mod( 'global_layout_type', gutenbee_global_layout_type_default() ) ) {
		$classes[] = 'layout-boxed';
	}

	if ( is_customize_preview() ) {
		$classes[] = 'is-customize-preview';
	}

	return $classes;
}

/**
 * Modify the excerpt length according to the user option.
 */
add_filter( 'excerpt_length', 'gutenbee_excerpt_length' );
function gutenbee_excerpt_length( $length ) {
	return get_theme_mod( 'blog_content_archives_excerpt_length', gutenbee_customizer_defaults( 'blog_content_archives_excerpt_length' ) );
}

/**
 * Get correct logo (normal/alternative) depending on header type.
 * Also handles case where an alternative logo is set but a normal one isn't.
 */
add_filter( 'get_custom_logo', 'gutenbee_get_custom_logo_filter', 10, 2 );
function gutenbee_get_custom_logo_filter( $html, $blog_id ) {
	$logo     = get_theme_mod( 'custom_logo' );
	$logo_alt = get_theme_mod( 'site_identity_custom_logo_alt' );

	if ( empty( $logo ) && empty( $logo_alt ) ) {
		return $html;
	}

	// $html contains display:none when a logo is not set but we're previewing in customizer.
	// Both cases mean that $logo is not set.
	if ( empty( $html ) || false !== strpos( $html, 'display:none' ) ) {
		$custom_logo_id = $logo_alt;

		// The following code is identical to get_custom_logo() as of WP 5.0.3
		$custom_logo_attr = array(
			'class'    => 'custom-logo custom-logo-alt',
			'itemprop' => 'logo',
		);

		/*
		 * If the logo alt attribute is empty, get the site title and explicitly
		 * pass it to the attributes used by wp_get_attachment_image().
		 */
		$image_alt = get_post_meta( $custom_logo_id, '_wp_attachment_image_alt', true );
		if ( empty( $image_alt ) ) {
			$custom_logo_attr['alt'] = get_bloginfo( 'name', 'display' );
		}

		/*
		 * If the alt attribute is not empty, there's no need to explicitly pass
		 * it because wp_get_attachment_image() already adds the alt attribute.
		 */
		$html = sprintf( '<a href="%1$s" class="custom-logo-link" rel="home" itemprop="url">%2$s</a>',
			esc_url( home_url( '/' ) ),
			wp_get_attachment_image( $custom_logo_id, 'full', false, $custom_logo_attr )
		);
	} elseif ( $logo && $logo_alt ) {
		// We have both a logo and logo_alt. Show the correct one depending on header type.
		if ( 'transparent' === gutenbee_get_mod( 'header_layout_type', gutenbee_customizer_defaults( 'header_layout_type' ) ) ) {
			$alt_url = wp_get_attachment_image_url( $logo_alt, 'full' );

			$html = preg_replace( '/src="(.*?)"/', 'src="' . esc_url( $alt_url ) . '"', $html );
		}

		$sticky_logo = 'normal';
		if ( gutenbee_get_mod( 'header_layout_menu_sticky_alt_logo', gutenbee_customizer_defaults( 'header_layout_menu_sticky_alt_logo' ) ) ) {
			$sticky_logo = 'alt';
		}

		$data_attributes = sprintf( 'data-logo="%s" data-logo-alt="%s" data-sticky-logo="%s" ',
			esc_url( wp_get_attachment_image_url( $logo, 'full' ) ),
			esc_url( wp_get_attachment_image_url( $logo_alt, 'full' ) ),
			esc_attr( $sticky_logo )
		);

		$html = str_replace( '<img ', '<img ' . $data_attributes, $html );
	}

	// Strip srcset and sizes attributes.
	$html = preg_replace( '/srcset="(.*?)"/', '', $html );
	$html = preg_replace( '/sizes="(.*?)"/', '', $html );

	return $html;
}

/**
 * Enable lightbox in content and comments, if applicable.
 */
add_filter( 'the_content', 'gutenbee_lightbox_add_rel', 12 );
add_filter( 'get_comment_text', 'gutenbee_lightbox_add_rel' );
function gutenbee_lightbox_add_rel( $content ) {
	if ( get_theme_mod( 'utilities_lightbox', gutenbee_customizer_defaults( 'utilities_lightbox' ) ) ) {
		global $post;
		$pattern     = "/<a(.*?)href=('|\")([^>]*).(bmp|gif|jpeg|jpg|png)('|\")(.*?)>(.*?)<\/a>/i";
		$replacement = '<a$1href=$2$3.$4$5 data-lightbox="gal[' . $post->ID . ']"$6>$7</a>';
		$content     = preg_replace( $pattern, $replacement, $content );
	}

	return $content;
}

/**
 * Enable lightbox captions, if applicable.
 */
add_filter( 'wp_get_attachment_link', 'gutenbee_wp_get_attachment_link_lightbox_caption', 10, 6 );
function gutenbee_wp_get_attachment_link_lightbox_caption( $html, $id, $size, $permalink, $icon, $text ) {
	if ( get_theme_mod( 'utilities_lightbox', gutenbee_customizer_defaults( 'utilities_lightbox' ) ) && false === $permalink ) {
		$found = preg_match( '#(<a.*?>)<img.*?></a>#', $html, $matches );
		if ( $found ) {
			$found_title = preg_match( '#title=([\'"])(.*?)\1#', $matches[1], $title_matches );

			// Only continue if title attribute doesn't exist.
			if ( 0 === $found_title ) {
				$caption = gutenbee_get_image_lightbox_caption( $id );

				if ( $caption ) {
					$new_a = $matches[1];
					$new_a = rtrim( $new_a, '>' );
					$new_a = $new_a . ' title="' . esc_attr( $caption ) . '">';

					$html = str_replace( $matches[1], $new_a, $html );
				}
			}
		}
	}

	return $html;
}

/**
 * Remove extraneous div and make the ul attributes of wp_page_menu() identical to wp_nav_menu().
 */
add_filter( 'wp_page_menu', 'gutenbee_wp_page_menu', 10, 2 );
function gutenbee_wp_page_menu( $menu, $args ) {
	$menu = preg_replace( '#^<div .*?>#', '', $menu, 1 );
	$menu = preg_replace( '#</div>$#', '', $menu, 1 );
	$menu = preg_replace( '#^<ul>#', '<ul id="' . esc_attr( $args['menu_id'] ) . '" class="' . esc_attr( $args['menu_class'] ) . '">', $menu, 1 );
	return $menu;
}

/**
 * Wraps the core's archive widget's post counts in span.ci-count
 */
add_filter( 'get_archives_link', 'gutenbee_wrap_archive_widget_post_counts_in_span', 10, 2 );
function gutenbee_wrap_archive_widget_post_counts_in_span( $output ) {
	$output = preg_replace_callback( '#(<li>.*?<a.*?>.*?</a>.*?)&nbsp;\((.*?)\)(.*?</li>)#', function( $matches ) {
		return sprintf( '%s <span class="ci-count">%s</span>%s',
			$matches[1],
			$matches[2],
			$matches[3]
		);
	}, $output );

	return $output;
}

/**
 * Wraps the core's category widget's post counts in span.ci-count
 */
add_filter( 'wp_list_categories', 'gutenbee_wrap_category_widget_post_counts_in_span', 10, 2 );
function gutenbee_wrap_category_widget_post_counts_in_span( $output, $args ) {
	if ( ! isset( $args['show_count'] ) || 0 === intval( $args['show_count'] ) ) {
		return $output;
	}

	if ( ! empty( $args['walker'] ) && is_object( $args['walker'] ) && 'WC_Product_Cat_List_Walker' === get_class( $args['walker'] ) ) {
		// Capture only the span and whatever is *inside* the parentheses.
		$output = preg_replace( '#(<span.*?>)\s*?\((.*?)\)\s*?(</span>)#', '\\1\\2\\3', $output );
	} else {
		$output = preg_replace_callback( '#(<a.*?>)\s*?\((.*?)\)#', function( $matches ) {
			return sprintf( '%s <span class="ci-count">%s</span>',
				$matches[1],
				$matches[2]
			);
		}, $output );
	}

	return $output;
}

/**
 * Add default wp_link_pages() arguments.
 */
add_filter( 'wp_link_pages_args', 'gutenbee_wp_link_pages_default_args' );
function gutenbee_wp_link_pages_default_args( $args ) {
	$new_args = array(
		'before'      => '<div class="page-links">' . esc_html__( 'Pages:', 'gutenbee' ),
		'after'       => '</div>',
		'link_before' => '<span class="page-number">',
		'link_after'  => '</span>',
	);

	$args = wp_parse_args( $new_args, $args );

	return $args;
}

/**
 * Add custom style formats in the editor's style dropdown.
 */
add_filter( 'tiny_mce_before_init', 'gutenbee_insert_wp_editor_formats' );
function gutenbee_insert_wp_editor_formats( $init_array ) {
	$style_formats = array(
		array(
			'title'   => esc_html__( 'Intro text (big text)', 'gutenbee' ),
			'block'   => 'div',
			'classes' => 'entry-content-intro',
			'wrapper' => true,
		),
		array(
			'title'   => esc_html__( '2 Column Text', 'gutenbee' ),
			'block'   => 'div',
			'classes' => 'entry-content-column-split',
			'wrapper' => true,
		),
	);

	$init_array['style_formats'] = wp_json_encode( $style_formats );

	return $init_array;
}

/**
 * Enable the editor's style dropdown.
 */
add_filter( 'mce_buttons_2', 'gutenbee_mce_buttons_2' );
function gutenbee_mce_buttons_2( $buttons ) {
	array_unshift( $buttons, 'styleselect' );

	return $buttons;
}

/**
 * Filter out menu classes that falsely show the blog page as an ancestor of custom post type singulars.
 * 'page_css_class' filter applies to wp_page_menu()
 * 'nav_menu_css_class' filter applies to wp_nav_menu()
 */
add_filter( 'page_css_class', 'gutenbee_remove_cpt_parent_class_from_blog' );
add_filter( 'nav_menu_css_class', 'gutenbee_remove_cpt_parent_class_from_blog' );
function gutenbee_remove_cpt_parent_class_from_blog( $classes ) {
	$post_types = get_post_types( array(
		'public'   => true,
		'_builtin' => false,
	), 'names' );

	if ( is_singular( $post_types ) ) {
		return array_filter( $classes, function( $class ) {
			// Check for current page classes, return false if they exist.
			if ( in_array( $class, array( 'current_page_item', 'current_page_parent', 'current_page_ancestor' ), true ) ) {
				return false;
			}

			return true;
		} );
	} else {
		return $classes;
	}
}

add_filter( 'embed_oembed_html', 'gutenbee_oembed_responsive_wrapper', 10, 4 );
function gutenbee_oembed_responsive_wrapper( $cache, $url, $attr, $post_ID ) {
	if ( empty( $cache ) ) {
		return $cache;
	}

	$url_patterns = array(
		'youtube.com',
		'youtu.be',
		'youtube-nocookie.com', // This doesn't seem to embed anything.
		'vimeo.com',
		'dailymotion.com',
		'dai.ly', // This doesn't seem to embed anything.
		'hulu.com',
		'wordpress.tv',
		'slideshare.net',
	);

	$match = false;

	foreach ( $url_patterns as $url_pattern ) {
		$pattern = 'https?://.*?' . preg_quote( $url_pattern, '#' );
		if ( preg_match( '#' . $pattern . '#', $url ) ) {
			$match = true;
			break;
		}
	}

	if ( $match ) {
		$cache = '<div class="gutenbee-responsive-embed">' . $cache . '</div>';
	}

	return $cache;
}

/**
 * Include a skip to content link at the top of the page so that users can bypass the menu.
 */
add_action( 'wp_body_open', 'gutenbee_skip_link', 5 );
function gutenbee_skip_link() {
	?><div><a class="skip-link sr-only sr-only-focusable" href="#site-content"><?php esc_html_e( 'Skip to the content', 'gutenbee' ); ?></a></div><?php
}
