<?php
/**
 * Returns a set of related posts, or the arguments needed for such a query.
 *
 * @uses wp_parse_args()
 * @uses get_post_type()
 * @uses get_post()
 * @uses get_object_taxonomies()
 * @uses get_the_terms()
 * @uses wp_list_pluck()
 *
 * @param int $post_id A post ID to get related posts for.
 * @param int $related_count The number of related posts to return.
 * @param array $args Array of arguments to change the default behavior.
 * @return object|array A WP_Query object with the results, or an array with the query arguments.
 */
function gutenbee_get_related_posts( $post_id, $related_count, $args = array() ) {
	$args = wp_parse_args( (array) $args, array(
		'orderby' => 'rand',
		'return'  => 'query', // Valid values are: 'query' (WP_Query object), 'array' (the arguments array)
	) );

	$post_type = get_post_type( $post_id );
	$post      = get_post( $post_id );

	$tax_query  = array();
	$taxonomies = get_object_taxonomies( $post, 'names' );

	foreach ( $taxonomies as $taxonomy ) {
		$terms = get_the_terms( $post_id, $taxonomy );
		if ( is_array( $terms ) && count( $terms ) > 0 ) {
			$term_list = wp_list_pluck( $terms, 'slug' );
			$term_list = array_values( $term_list );
			if ( ! empty( $term_list ) ) {
				$tax_query['tax_query'][] = array(
					'taxonomy' => $taxonomy,
					'field'    => 'slug',
					'terms'    => $term_list,
				);
			}
		}
	}

	if ( count( $taxonomies ) > 1 ) {
		$tax_query['tax_query']['relation'] = 'OR';
	}

	$query_args = array(
		'post_type'      => $post_type,
		'posts_per_page' => $related_count,
		'post_status'    => 'publish',
		'post__not_in'   => array( $post_id ),
		'orderby'        => $args['orderby'],
	);

	if ( 'query' === $args['return'] ) {
		return new WP_Query( array_merge( $query_args, $tax_query ) );
	} else {
		return array_merge( $query_args, $tax_query );
	}
}


/**
 * Lightens/darkens a given colour (hex format), returning the altered colour in hex format.
 *
 * @see https://gist.github.com/stephenharris/5532899
 *
 * @param string $color Hexadecimal color value. May be 3 or 6 digits, with an optional leading # sign.
 * @param float $percent Decimal (0.2 = lighten by 20%, -0.4 = darken by 40%)
 *
 * @return string Lightened/Darkened colour as hexadecimal (with hash)
 */
function gutenbee_color_luminance( $color, $percent ) {
	// Remove # if provided
	$color = ltrim( $color, '#' );

	// Validate hex string.
	$hex     = preg_replace( '/[^0-9a-f]/i', '', $color );
	$new_hex = '#';

	$percent = floatval( $percent );

	if ( strlen( $hex ) < 6 ) {
		$hex = $hex[0] + $hex[0] + $hex[1] + $hex[1] + $hex[2] + $hex[2];
	}

	// Convert to decimal and change luminosity.
	for ( $i = 0; $i < 3; $i ++ ) {
		$dec = hexdec( substr( $hex, $i * 2, 2 ) );
		$dec = min( max( 0, $dec + $dec * $percent ), 255 );

		$new_hex .= str_pad( dechex( (int) $dec ), 2, 0, STR_PAD_LEFT );
	}

	return $new_hex;
}

/**
 * Converts hexadecimal color value to rgb(a) format.
 *
 * @param string $color Hexadecimal color value. May be 3 or 6 digits, with an optional leading # sign.
 * @param float|bool $opacity Opacity level 0-1 (decimal) or false to disable.
 *
 * @return string
 */
function gutenbee_hex2rgba( $color, $opacity = false ) {

	$default = 'rgb(0,0,0)';

	// Return default if no color provided
	if ( empty( $color ) ) {
		return $default;
	}

	// Remove # if provided
	$color = ltrim( $color, '#' );

	// Check if color has 6 or 3 characters and get values
	if ( strlen( $color ) === 6 ) {
		$hex = array( $color[0] . $color[1], $color[2] . $color[3], $color[4] . $color[5] );
	} elseif ( strlen( $color ) === 3 ) {
		$hex = array( $color[0] . $color[0], $color[1] . $color[1], $color[2] . $color[2] );
	} else {
		return $default;
	}

	$rgb = array_map( 'hexdec', $hex );

	if ( false !== $opacity ) {
		$opacity = abs( floatval( $opacity ) );
		if ( $opacity > 1 ) {
			$opacity = 1.0;
		}
		$output = 'rgba(' . implode( ',', $rgb ) . ',' . $opacity . ')';
	} else {
		$output = 'rgb(' . implode( ',', $rgb ) . ')';
	}

	return $output;
}

/**
 * Returns the caption of an image, to be used in a lightbox.
 *
 * @uses get_post_thumbnail_id()
 * @uses wp_prepare_attachment_for_js()
 *
 * @param int|false $image_id The image's attachment ID.
 *
 * @return string
 */
function gutenbee_get_image_lightbox_caption( $image_id = false ) {
	if ( false === $image_id ) {
		$image_id = get_post_thumbnail_id();
	}

	$lightbox_caption = '';

	$attachment = wp_prepare_attachment_for_js( $image_id );

	if ( is_array( $attachment ) ) {
		$field = apply_filters( 'gutenbee_image_lightbox_caption_field', 'caption', $image_id, $attachment );

		if ( array_key_exists( $field, $attachment ) ) {
			$lightbox_caption = $attachment[ $field ];
		}
	}

	return $lightbox_caption;
}

/**
 * Retrieves a theme_mod value, or it's overridden value by post/term meta.
 * Assumes that an empty string value means we want to fall back to the customizer value.
 *
 * @param string $name    Option name
 * @param bool   $default Default value to pass to get_theme_mod()
 *
 * @return mixed
 */
function gutenbee_get_mod( $name, $default = false ) {
	$object_id = false;

	if ( false === $object_id ) {
		if ( is_singular() ) {
			$object_id = get_the_ID();
		} elseif ( is_home() && gutenbee_blog_is_page() ) {
			$object_id = gutenbee_get_blog_page_id();
		} elseif ( is_tax() || is_category() || is_tag() ) {
			$object_id = get_queried_object_id();
		} elseif ( class_exists( 'WooCommerce' ) && is_shop() ) {
			$object_id = wc_get_page_id( 'shop' );
		}
	}

	$value      = $default;
	$customizer = get_theme_mod( $name, $default );
	$meta_keys  = array();

	if ( is_singular() || ( is_home() && gutenbee_blog_is_page() ) ) {
		$meta_keys = get_post_custom_keys( $object_id );
		if ( is_null( $meta_keys ) ) {
			$meta_keys = array();
		}
	}

	$case = false;
	if ( is_singular() && in_array( $name, $meta_keys, true ) ) {
		$case = 'singular';
	} elseif ( is_home() && gutenbee_blog_is_page() && in_array( $name, $meta_keys, true ) ) {
		$case = 'blog_page';
	} elseif ( is_tax() || is_category() || is_tag() ) {
		$case = 'term';
	} elseif ( class_exists( 'WooCommerce' ) && is_shop() ) {
		$case = 'shop';
	} else {
		$case = false;
	}

	switch ( $case ) {
		case 'singular':
		case 'blog_page':
		case 'shop':
			$value = get_post_meta( $object_id, $name, true );
			if ( false === $value || '' === $value ) {
				$value = $customizer;
			} elseif ( is_array( $value ) && is_array( $default ) ) {
				// Make sure the arrays contain identical keys, otherwise the comparison is not meaningful.
				$value = shortcode_atts( $default, $value );

				ksort( $value );
				ksort( $default );
				if ( $value === $default ) {
					$value = $customizer;
				}
			}
			break;
		case 'term':
			$value = get_term_meta( $object_id, $name, true );
			if ( false === $value || '' === $value ) {
				$value = $customizer;
			}
			break;
		default:
			$value = $customizer;
	}

	return apply_filters( "gutenbee_mod_{$name}", $value, $name, $default, $object_id );
}

/**
 * Retrieve a meta field of a post, returning a default value if no stored value is found.
 *
 * @param int    $post_id   Post ID.
 * @param string $fieldname The meta key to retrieve.
 * @param bool   $default   Value to return if no stored value is found.
 *
 * @return mixed
 */
function gutenbee_get_post_meta( $post_id, $fieldname, $default = false ) {
	if ( metadata_exists( 'post', $post_id, $fieldname ) ) {
		$value = get_post_meta( $post_id, $fieldname, true );
	} else {
		$value = $default;
	}

	return $value;
}

/**
 * Determines if the blog section of the site is configured as a page.
 *
 * @return bool
 */
function gutenbee_blog_is_page() {
	$show_on_front  = get_option( 'show_on_front' );
	$page_for_posts = get_option( 'page_for_posts' );

	if ( 'page' === $show_on_front && $page_for_posts ) {
		return true;
	}

	return false;
}

/**
 * Returns the page ID that's configured as the blog page.
 *
 * @return int|false
 */
function gutenbee_get_blog_page_id() {
	$show_on_front  = get_option( 'show_on_front' );
	$page_for_posts = get_option( 'page_for_posts' );

	if ( 'page' === $show_on_front && $page_for_posts ) {
		return intval( $page_for_posts );
	}

	return false;
}

/**
 * Determines if the current page is one that the theme considers to be a shop page with a sidebar.
 *
 * @return bool
 */
function gutenbee_is_woocommerce_with_sidebar() {
	if ( ! class_exists( 'WooCommerce' ) ) {
		return false;
	}

	$return = false;

	if ( is_shop() || is_product_taxonomy() ) {
		$return = true;
	}

	return (bool) apply_filters( 'gutenbee_is_woocommerce_with_sidebar', $return );
}

/**
 * Determines if the current page is one that the theme considers to be a shop page without a sidebar.
 *
 * @return bool
 */
function gutenbee_is_woocommerce_without_sidebar() {
	if ( ! class_exists( 'WooCommerce' ) ) {
		return false;
	}

	$return = false;

	if ( is_product() || is_cart() || is_checkout() || is_account_page() ) {
		$return = true;
	}

	return (bool) apply_filters( 'gutenbee_is_woocommerce_without_sidebar', $return );
}

/**
 * Conditionally returns a Javascript/CSS asset's version number.
 *
 * When the site is in debug mode, the normal asset's version is returned.
 * When it's not in debug mode, the theme's version is returned, so that caches can be invalidated on theme updates.
 *
 * @param bool $version The version string of the asset.
 *
 * @return false|string Theme version if SCRIPT_DEBUG or WP_DEBUG are enabled. Otherwise, $version is returned.
 */
function gutenbee_asset_version( $version = false ) {
	static $theme_version = false;

	if ( ! $theme_version ) {
		$theme = wp_get_theme();
		if ( is_child_theme() ) {
			$theme_version = $theme->parent()->get( 'Version' ) . '-' . $theme->get( 'Version' );
		} else {
			$theme_version = $theme->get( 'Version' );
		}
	}

	if ( $version ) {
		if ( ( defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG ) ||
			( defined( 'WP_DEBUG' ) && WP_DEBUG )
		) {
			return $version;
		}
	}

	return $theme_version;
}

/**
 * Generic fallback callback for the main menu.
 *
 * Displays a few useful links, in contrast to the default wp_page_menu() which may flood the menu area.
 *
 * @param array $args
 *
 * @return void|string
 */
function gutenbee_main_menu_fallback( $args ) {
	$id    = ! empty( $args['menu_id'] ) ? $args['menu_id'] : 'menu-' . $args['theme_location'];
	$class = $args['menu_class'];

	$items = apply_filters( 'gutenbee_main_menu_fallback_items', array(
		home_url( '/' )              => __( 'Home', 'gutenbee' ),
		admin_url( 'nav-menus.php' ) => __( 'Set primary menu', 'gutenbee' ),
	) );

	$items_html = '';
	if ( $items ) {
		foreach ( $items as $item_url => $item_text ) {
			$items_html .= sprintf( '<li><a href="%1$s">%2$s</a></li>', esc_url( $item_url ), esc_html( $item_text ) );
		}
	}

	if ( empty( $items_html ) ) {
		return false;
	}

	$menu_html = sprintf( $args['items_wrap'], esc_attr( $id ), esc_attr( $class ), $items_html );

	if ( $args['echo'] ) {
		echo wp_kses_post( $menu_html );
	} else {
		return $menu_html;
	}
}

/**
 * Outputs a preconnect link that helps performance of Google Fonts.
 * Hooked conditionally on 'wp_head'.
 */
function gutenbee_head_preconnect_google_fonts() {
	?><link rel="preconnect" href="https://fonts.gstatic.com/" crossorigin><?php
}

// Make sure this function is defined, even if the plugin is disabled.
if ( ! function_exists( 'gutenbee_documentation_url' ) ) {
	/**
	 * Returns the URL to the theme's documentation page.
	 *
	 * @return string
	 */
	function gutenbee_documentation_url() {
		// TODO: Make sure this is correct (or will be correct after gutenbee replacements). Adjust by hand if necessary.
		$url = 'https://gutenbee.com/docs/';

		return apply_filters( 'gutenbee_documentation_url', $url );
	}
}
