<?php
/**
 * Generic sanitization functions.
 */

/**
 * Sanitizes integer input while differentiating zero from empty string.
 *
 * @param int|string $input Input value to sanitize.
 * @return int|string Integer value, 0, or an empty string otherwise.
 */
function gutenbee_sanitize_intval_or_empty( $input ) {
	if ( false === $input || '' === $input || is_null( $input ) ) {
		return '';
	}

	if ( 0 === intval( $input ) ) {
		return 0;
	}

	return intval( $input );
}

/**
 * Sanitizes float input while differentiating zero from empty string.
 *
 * @param int|string $input Input value to sanitize.
 * @return int|string Integer value, 0, or an empty string otherwise.
 */
function gutenbee_sanitize_floatval_or_empty( $input ) {
	if ( false === $input || '' === $input || is_null( $input ) ) {
		return '';
	}

	if ( 0 === floatval( $input ) ) {
		return 0;
	}

	return floatval( $input );
}

/**
 * Return a list of allowed tags and attributes for a given context.
 *
 * @param string $context The context for which to retrieve tags.
 *                        Currently available contexts: guide
 * @return array List of allowed tags and their allowed attributes.
 */
function gutenbee_get_allowed_tags( $context = '' ) {
	$allowed = array(
		'a'       => array(
			'href'   => true,
			'title'  => true,
			'class'  => true,
			'target' => true,
			'rel'    => true,
		),
		'abbr'    => array( 'title' => true ),
		'acronym' => array( 'title' => true ),
		'b'       => array( 'class' => true ),
		'br'      => array(),
		'code'    => array( 'class' => true ),
		'em'      => array( 'class' => true ),
		'i'       => array( 'class' => true ),
		'img'     => array(
			'alt'    => true,
			'class'  => true,
			'src'    => true,
			'width'  => true,
			'height' => true,
		),
		'li'      => array( 'class' => true ),
		'ol'      => array( 'class' => true ),
		'p'       => array( 'class' => true ),
		'pre'     => array( 'class' => true ),
		'span'    => array( 'class' => true ),
		'strong'  => array( 'class' => true ),
		'ul'      => array( 'class' => true ),
	);

	switch ( $context ) {
		case 'guide':
			unset( $allowed['p'] );
			break;
		default:
			break;
	}

	return apply_filters( 'gutenbee_get_allowed_tags', $allowed, $context );
}

function gutenbee_get_image_repeat_choices() {
	return apply_filters( 'gutenbee_image_repeat_choices', array(
		'no-repeat' => esc_html__( 'No repeat', 'gutenbee' ),
		'repeat'    => esc_html__( 'Tile', 'gutenbee' ),
		'repeat-x'  => esc_html__( 'Tile Horizontally', 'gutenbee' ),
		'repeat-y'  => esc_html__( 'Tile Vertically', 'gutenbee' ),
	) );
}

function gutenbee_sanitize_image_repeat( $value ) {
	$choices = gutenbee_get_image_repeat_choices();
	if ( array_key_exists( $value, $choices ) ) {
		return $value;
	}

	return apply_filters( 'gutenbee_sanitize_image_repeat_default', 'no-repeat' );
}

function gutenbee_get_image_position_choices() {
	return apply_filters( 'gutenbee_image_position_choices', array(
		'left top'      => esc_html__( 'Top Left', 'gutenbee' ),
		'center top'    => esc_html__( 'Top Center', 'gutenbee' ),
		'right top'     => esc_html__( 'Top Right', 'gutenbee' ),
		'left center'   => esc_html__( 'Center Left', 'gutenbee' ),
		'center center' => esc_html__( 'Center Center', 'gutenbee' ),
		'right center'  => esc_html__( 'Center Right', 'gutenbee' ),
		'left bottom'   => esc_html__( 'Bottom Left', 'gutenbee' ),
		'center bottom' => esc_html__( 'Bottom Center', 'gutenbee' ),
		'right bottom'  => esc_html__( 'Bottom Right', 'gutenbee' ),
	) );
}
function gutenbee_sanitize_image_position( $value ) {
	$choices = gutenbee_get_image_position_choices();
	if ( array_key_exists( $value, $choices ) ) {
		return $value;
	}

	return apply_filters( 'gutenbee_sanitize_image_position_default', 'center top' );
}


function gutenbee_get_image_attachment_choices() {
	return apply_filters( 'gutenbee_image_attachment_choices', array(
		'scroll' => esc_html__( 'Scroll', 'gutenbee' ),
		'fixed'  => esc_html__( 'Fixed', 'gutenbee' ),
	) );
}

function gutenbee_sanitize_image_attachment( $value ) {
	$choices = gutenbee_get_image_attachment_choices();
	if ( array_key_exists( $value, $choices ) ) {
		return $value;
	}

	return apply_filters( 'gutenbee_sanitize_image_attachment_default', 'scroll' );
}

function gutenbee_get_image_size_choices() {
	return apply_filters( 'gutenbee_image_size_choices', array(
		'cover'   => esc_html__( 'Cover', 'gutenbee' ),
		'contain' => esc_html__( 'Contain', 'gutenbee' ),
		'auto'    => esc_html__( 'Auto', 'gutenbee' ),
	) );
}

function gutenbee_sanitize_image_size( $value ) {
	$choices = gutenbee_get_image_size_choices();
	if ( array_key_exists( $value, $choices ) ) {
		return $value;
	}

	return apply_filters( 'gutenbee_sanitize_image_size_default', 'cover' );
}

function gutenbee_sanitize_rgba_color( $str, $return_hash = true, $return_fail = '' ) {
	if ( false === $str || empty( $str ) || 'false' === $str ) {
		return $return_fail;
	}

	// Allow keywords and predefined colors
	if ( in_array( $str, array( 'transparent', 'initial', 'inherit', 'black', 'silver', 'gray', 'grey', 'white', 'maroon', 'red', 'purple', 'fuchsia', 'green', 'lime', 'olive', 'yellow', 'navy', 'blue', 'teal', 'aqua', 'orange', 'aliceblue', 'antiquewhite', 'aquamarine', 'azure', 'beige', 'bisque', 'blanchedalmond', 'blueviolet', 'brown', 'burlywood', 'cadetblue', 'chartreuse', 'chocolate', 'coral', 'cornflowerblue', 'cornsilk', 'crimson', 'darkblue', 'darkcyan', 'darkgoldenrod', 'darkgray', 'darkgrey', 'darkgreen', 'darkkhaki', 'darkmagenta', 'darkolivegreen', 'darkorange', 'darkorchid', 'darkred', 'darksalmon', 'darkseagreen', 'darkslateblue', 'darkslategray', 'darkslategrey', 'darkturquoise', 'darkviolet', 'deeppink', 'deepskyblue', 'dimgray', 'dimgrey', 'dodgerblue', 'firebrick', 'floralwhite', 'forestgreen', 'gainsboro', 'ghostwhite', 'gold', 'goldenrod', 'greenyellow', 'grey', 'honeydew', 'hotpink', 'indianred', 'indigo', 'ivory', 'khaki', 'lavender', 'lavenderblush', 'lawngreen', 'lemonchiffon', 'lightblue', 'lightcoral', 'lightcyan', 'lightgoldenrodyellow', 'lightgray', 'lightgreen', 'lightgrey', 'lightpink', 'lightsalmon', 'lightseagreen', 'lightskyblue', 'lightslategray', 'lightslategrey', 'lightsteelblue', 'lightyellow', 'limegreen', 'linen', 'mediumaquamarine', 'mediumblue', 'mediumorchid', 'mediumpurple', 'mediumseagreen', 'mediumslateblue', 'mediumspringgreen', 'mediumturquoise', 'mediumvioletred', 'midnightblue', 'mintcream', 'mistyrose', 'moccasin', 'navajowhite', 'oldlace', 'olivedrab', 'orangered', 'orchid', 'palegoldenrod', 'palegreen', 'paleturquoise', 'palevioletred', 'papayawhip', 'peachpuff', 'peru', 'pink', 'plum', 'powderblue', 'rosybrown', 'royalblue', 'saddlebrown', 'salmon', 'sandybrown', 'seagreen', 'seashell', 'sienna', 'skyblue', 'slateblue', 'slategray', 'slategrey', 'snow', 'springgreen', 'steelblue', 'tan', 'thistle', 'tomato', 'turquoise', 'violet', 'wheat', 'whitesmoke', 'yellowgreen', 'rebeccapurple' ), true ) ) {
		return $str;
	}

	preg_match( '/rgba\(\s*(\d{1,3}\.?\d*\%?)\s*,\s*(\d{1,3}\.?\d*\%?)\s*,\s*(\d{1,3}\.?\d*\%?)\s*,\s*(\d{1}\.?\d*\%?)\s*\)/', $str, $rgba_matches );
	if ( ! empty( $rgba_matches ) && 5 === count( $rgba_matches ) ) {
		for ( $i = 1; $i < 4; $i++ ) {
			if ( strpos( $rgba_matches[ $i ], '%' ) !== false ) {
				$rgba_matches[ $i ] = gutenbee_sanitize_0_100_percent( $rgba_matches[ $i ] );
			} else {
				$rgba_matches[ $i ] = gutenbee_sanitize_0_255( $rgba_matches[ $i ] );
			}
		}
		$rgba_matches[4] = gutenbee_sanitize_0_1_opacity( $rgba_matches[ $i ] );
		return sprintf( 'rgba(%s, %s, %s, %s)', $rgba_matches[1], $rgba_matches[2], $rgba_matches[3], $rgba_matches[4] );
	}

	// Not a color function either. Let's see if it's a hex color.

	// Include the hash if not there.
	// The regex below depends on in.
	$str = '#' . ltrim( $str, '#' );

	preg_match( '/(#)([0-9a-fA-F]{6})/', $str, $matches );

	if ( 3 === count( $matches ) ) {
		if ( $return_hash ) {
			return $matches[1] . $matches[2];
		} else {
			return $matches[2];
		}
	}

	return $return_fail;
}

function gutenbee_sanitize_0_100_percent( $val ) {
	$val = str_replace( '%', '', $val );
	if ( floatval( $val ) > 100 ) {
		$val = 100;
	} elseif ( floatval( $val ) < 0 ) {
		$val = 0;
	}

	return floatval( $val ) . '%';
}

function gutenbee_sanitize_0_255( $val ) {
	if ( intval( $val ) > 255 ) {
		$val = 255;
	} elseif ( intval( $val ) < 0 ) {
		$val = 0;
	}

	return intval( $val );
}

function gutenbee_sanitize_0_1_opacity( $val ) {
	if ( floatval( $val ) > 1 ) {
		$val = 1;
	} elseif ( floatval( $val ) < 0 ) {
		$val = 0;
	}

	return floatval( $val );
}

function gutenbee_global_layout_type_choices() {
	$choices = array(
		'full_width' => __( 'Full width', 'gutenbee' ),
		'boxed'      => __( 'Boxed', 'gutenbee' ),
	);

	return apply_filters( 'gutenbee_global_layout_type_choices', $choices );
}

function gutenbee_global_layout_type_default() {
	return apply_filters( 'gutenbee_global_layout_type_default', gutenbee_customizer_defaults( 'global_layout_type' ) );
}

function gutenbee_sanitize_global_layout_type( $value ) {
	$choices = gutenbee_global_layout_type_choices();
	if ( array_key_exists( $value, $choices ) ) {
		return $value;
	}

	return gutenbee_global_layout_type_default();
}



/**
 * Base set of theme's page layouts. Individual options may add/remove elements as needed.
 *
 * @return array
 */
function gutenbee_get_layout_types() {
	return apply_filters( 'gutenbee_layout_types', array(
		'content_sidebar'  => __( 'Content / Sidebar', 'gutenbee' ),
		'sidebar_content'  => __( 'Sidebar / Content', 'gutenbee' ),
		'fullwidth'        => __( 'Full width', 'gutenbee' ),
		'fullwidth_narrow' => __( 'Full width narrow', 'gutenbee' ),
	) );
}



function gutenbee_global_layout_pages_layout_type_choices() {
	$choices = gutenbee_get_layout_types();

	return apply_filters( 'gutenbee_global_layout_pages_layout_type_choices', $choices );
}

function gutenbee_global_layout_pages_layout_type_default() {
	return apply_filters( 'gutenbee_global_layout_pages_layout_type_default', gutenbee_customizer_defaults( 'global_layout_pages_layout_type' ) );
}

function gutenbee_sanitize_global_layout_pages_layout_type( $value ) {
	$choices = gutenbee_global_layout_pages_layout_type_choices();
	if ( array_key_exists( $value, $choices ) ) {
		return $value;
	}

	return gutenbee_global_layout_pages_layout_type_default();
}



function gutenbee_blog_layout_archives_layout_type_choices() {
	$choices = gutenbee_get_layout_types();

	return apply_filters( 'gutenbee_blog_layout_archives_layout_type_choices', $choices );
}

function gutenbee_blog_layout_archives_layout_type_default() {
	return apply_filters( 'gutenbee_blog_layout_archives_layout_type_default', gutenbee_customizer_defaults( 'blog_layout_archives_layout_type' ) );
}

function gutenbee_sanitize_blog_layout_archives_layout_type( $value ) {
	$choices = gutenbee_blog_layout_archives_layout_type_choices();
	if ( array_key_exists( $value, $choices ) ) {
		return $value;
	}

	return gutenbee_blog_layout_archives_layout_type_default();
}



function gutenbee_blog_layout_posts_layout_type_choices() {
	/* translators: %d is a number of columns. */
	$nooped = _n_noop( '%d Column', '%d Columns', 'gutenbee' );

	$choices = array(
		'1col-horz' => __( '1 Column horizontal', 'gutenbee' ),
		'1col'      => sprintf( translate_nooped_plural( $nooped, 1, 'gutenbee' ), 1 ),
		'2col'      => sprintf( translate_nooped_plural( $nooped, 2, 'gutenbee' ), 2 ),
		'3col'      => sprintf( translate_nooped_plural( $nooped, 3, 'gutenbee' ), 3 ),
		'4col'      => sprintf( translate_nooped_plural( $nooped, 4, 'gutenbee' ), 4 ),
	);

	return apply_filters( 'gutenbee_blog_layout_posts_layout_type_choices', $choices );
}

function gutenbee_blog_layout_posts_layout_type_default() {
	return apply_filters( 'gutenbee_blog_layout_posts_layout_type_default', gutenbee_customizer_defaults( 'blog_layout_posts_layout_type' ) );
}

function gutenbee_sanitize_blog_layout_posts_layout_type( $value ) {
	$choices = gutenbee_blog_layout_posts_layout_type_choices();
	if ( array_key_exists( $value, $choices ) ) {
		return $value;
	}

	return gutenbee_blog_layout_posts_layout_type_default();
}



function gutenbee_blog_layout_single_related_columns_choices() {
	/* translators: %d is a number of columns. */
	$nooped = _n_noop( '%d Column', '%d Columns', 'gutenbee' );

	$choices = array(
		'2' => sprintf( translate_nooped_plural( $nooped, 2, 'gutenbee' ), 2 ),
		'3' => sprintf( translate_nooped_plural( $nooped, 3, 'gutenbee' ), 3 ),
		'4' => sprintf( translate_nooped_plural( $nooped, 4, 'gutenbee' ), 4 ),
	);

	return apply_filters( 'gutenbee_blog_layout_single_related_columns_choices', $choices );
}

function gutenbee_blog_layout_single_related_columns_default() {
	return apply_filters( 'gutenbee_blog_layout_single_related_columns_default', gutenbee_customizer_defaults( 'blog_layout_single_related_columns' ) );
}

function gutenbee_sanitize_blog_layout_single_related_columns( $value ) {
	$choices = gutenbee_blog_layout_single_related_columns_choices();
	if ( array_key_exists( $value, $choices ) ) {
		return $value;
	}

	return gutenbee_blog_layout_single_related_columns_default();
}



function gutenbee_archives_pagination_choices() {
	$choices = array(
		'numbers'  => _x( 'Numbers', 'pagination method', 'gutenbee' ),
		'prevnext' => _x( 'Previous / Next', 'pagination method', 'gutenbee' ),
	);

	return apply_filters( 'gutenbee_archives_pagination_choices', $choices );
}

function gutenbee_archives_pagination_default() {
	return apply_filters( 'gutenbee_archives_pagination_default', gutenbee_customizer_defaults( 'blog_layout_archives_pagination' ) );
}

function gutenbee_sanitize_archives_pagination( $value ) {
	$choices = gutenbee_archives_pagination_choices();
	if ( array_key_exists( $value, $choices ) ) {
		return $value;
	}

	return gutenbee_archives_pagination_default();
}



function gutenbee_header_layout_menu_type_choices() {
	$choices = array(
		'full_left'            => __( 'Menu Left', 'gutenbee' ),
		'full_center'          => __( 'Menu Center', 'gutenbee' ),
		'full_right'           => __( 'Menu Right', 'gutenbee' ),
		'split'                => __( 'Menu Split', 'gutenbee' ),
		'navbar_top_left'      => __( 'Menu Top Left', 'gutenbee' ),
		'navbar_top_center'    => __( 'Menu Top Center', 'gutenbee' ),
		'navbar_top_right'     => __( 'Menu Top Right', 'gutenbee' ),
		'navbar_bottom_left'   => __( 'Menu Bottom Left', 'gutenbee' ),
		'navbar_bottom_center' => __( 'Menu Bottom Center', 'gutenbee' ),
		'navbar_bottom_right'  => __( 'Menu Bottom Right', 'gutenbee' ),
	);

	return apply_filters( 'gutenbee_header_layout_menu_type_choices', $choices );
}

function gutenbee_sanitize_header_layout_menu_type( $value ) {
	$choices = gutenbee_header_layout_menu_type_choices();
	if ( array_key_exists( $value, $choices ) ) {
		return $value;
	}

	return gutenbee_customizer_defaults( 'header_layout_menu_type' );
}



function gutenbee_devices_visibility_choices() {
	$choices = array(
		'all'        => __( 'All Devices', 'gutenbee' ),
		'desktop'    => __( 'Desktop Only', 'gutenbee' ),
		'all_mobile' => __( 'Tablets & Mobile Devices', 'gutenbee' ),
		'mobile'     => __( 'Mobile Devices', 'gutenbee' ),
	);

	return apply_filters( 'gutenbee_devices_visibility_choices', $choices );
}

function gutenbee_devices_visibility_default() {
	return apply_filters( 'gutenbee_devices_visibility_default', 'all' );
}

function gutenbee_sanitize_devices_visibility( $value ) {
	$choices = gutenbee_devices_visibility_choices();
	if ( array_key_exists( $value, $choices ) ) {
		return $value;
	}

	return gutenbee_devices_visibility_default();
}



function gutenbee_get_devices_visibility_class( $devices ) {
	$class = '';

	switch ( $devices ) {
		case 'desktop':
			$class = 'hidden-lg-down';
			break;
		case 'all_mobile':
			$class = 'hidden-xl-up';
			break;
		case 'mobile':
			$class = 'hidden-md-up';
			break;
		case 'all':
		default:
			$class = '';
			break;
	}

	return apply_filters( 'gutenbee_devices_visibility_class', $class, $devices );
}



function gutenbee_header_layout_type_choices() {
	$choices = array(
		'normal'      => __( 'Normal', 'gutenbee' ),
		'transparent' => __( 'Transparent', 'gutenbee' ),
	);

	return apply_filters( 'gutenbee_header_layout_type_choices', $choices );
}

function gutenbee_sanitize_header_layout_type( $value ) {
	$choices = gutenbee_header_layout_type_choices();
	if ( array_key_exists( $value, $choices ) ) {
		return $value;
	}

	return gutenbee_customizer_defaults( 'header_layout_type' );
}

function gutenbee_header_layout_type_meta_choices() {
	$choices = array_merge( array(
		'' => __( 'Respect customizer setting', 'gutenbee' ),
	), gutenbee_header_layout_type_choices() );

	return apply_filters( 'gutenbee_header_layout_type_meta_choices', $choices );
}

function gutenbee_sanitize_header_layout_type_meta( $value ) {
	$choices = gutenbee_header_layout_type_meta_choices();
	if ( array_key_exists( $value, $choices ) ) {
		return $value;
	}

	return '';
}




function gutenbee_text_tag_choices() {
	$choices = array(
		'h1'   => __( 'Heading 1 (h1)', 'gutenbee' ),
		'h2'   => __( 'Heading 2 (h2)', 'gutenbee' ),
		'h3'   => __( 'Heading 3 (h3)', 'gutenbee' ),
		'h4'   => __( 'Heading 4 (h4)', 'gutenbee' ),
		'h5'   => __( 'Heading 5 (h5)', 'gutenbee' ),
		'h6'   => __( 'Heading 6 (h6)', 'gutenbee' ),
		'p'    => __( 'Paragraph (p)', 'gutenbee' ),
		'span' => __( 'Span (span)', 'gutenbee' ),
		'div'  => __( 'Division (div)', 'gutenbee' ),
	);

	return apply_filters( 'gutenbee_text_tag_choices', $choices );
}

function gutenbee_text_tag_default() {
	return apply_filters( 'gutenbee_text_tag_default', 'p' );
}

function gutenbee_sanitize_text_tag( $value ) {
	$choices = gutenbee_text_tag_choices();
	if ( array_key_exists( $value, $choices ) ) {
		return $value;
	}

	return gutenbee_text_tag_default();
}



function gutenbee_align_horizontal_choices() {
	return apply_filters( 'gutenbee_align_horizontal_choices', array(
		'left'   => esc_html__( 'Left', 'gutenbee' ),
		'center' => esc_html__( 'Center', 'gutenbee' ),
		'right'  => esc_html__( 'Right', 'gutenbee' ),
	) );
}

function gutenbee_sanitize_align_horizontal( $value ) {
	$choices = gutenbee_align_horizontal_choices();
	if ( array_key_exists( $value, $choices ) ) {
		return $value;
	}

	return apply_filters( 'gutenbee_sanitize_align_horizontal_default', 'center' );
}



function gutenbee_align_vertical_choices() {
	return apply_filters( 'gutenbee_align_vertical_choices', array(
		'top'    => esc_html__( 'Top', 'gutenbee' ),
		'middle' => esc_html__( 'Middle', 'gutenbee' ),
		'bottom' => esc_html__( 'Bottom', 'gutenbee' ),
	) );
}

function gutenbee_sanitize_align_vertical( $value ) {
	$choices = gutenbee_align_vertical_choices();
	if ( array_key_exists( $value, $choices ) ) {
		return $value;
	}

	return apply_filters( 'gutenbee_sanitize_align_vertical_default', 'center' );
}

function gutenbee_assets_minify_pack_choices() {
	return apply_filters( 'gutenbee_assets_minify_pack_choices', array(
		'none'        => esc_html__( 'None', 'gutenbee' ),
		'minify'      => esc_html__( 'Minify', 'gutenbee' ),
		'minify_pack' => esc_html__( 'Minify & Pack', 'gutenbee' ),
	) );
}

function gutenbee_sanitize_assets_minify_pack( $value ) {
	$choices = gutenbee_assets_minify_pack_choices();
	if ( array_key_exists( $value, $choices ) ) {
		return $value;
	}

	return apply_filters( 'gutenbee_sanitize_assets_minify_pack_default', 'minify_pack' );
}
